import re
from typing import Tuple, List, Dict, Optional
import requests


def parse_repo_url(url: str) -> Tuple[str, str, str]:
    """Decompose a full repo URL into 3 parts:
    - the organization base URL
    - the namespace (i.e. groups and subgroups
    - the name of the repository

    Examples
    --------
    >>> parse_repo_url('https://org.com/group/subgroup/repo')
    ('https://org.com', 'group/subgroup', 'repo')
    >>> parse_repo_url('https://org.com/gitlab/group/repo')
    ('https://org.com/gitlab', 'group', 'repo')
    """
    regex = re.compile(
        (
            "(?P<base>https://[^/]*(/gitlab|/projects)?)/"
            "(?P<namespace>([^/]*/)*)"
            "(?P<repo>[^/]*)$"
        ),
        re.IGNORECASE,
    )
    captured = re.match(regex, url).groupdict()
    base, namespace, repo = [captured[group] for group in ["base", "namespace", "repo"]]
    namespace = namespace.strip("/")
    return base, namespace, repo


def parse_group_url(url: str) -> Tuple[str, str]:
    """Decompose a full group URL into 2 parts:
    - the organization base URL
    - the namespace (i.e. groups and subgroups

    Examples
    --------
    >>> parse_group_url('https://org.com/group/subgroup')
    ('https://org.com', 'group/subgroup')
    >>> parse_group_url('https://org.com/gitlab/group')
    ('https://org.com/gitlab', 'group')
    """
    regex = re.compile(
        (
            "(?P<base>https://[^/]*(/gitlab|/projects)?)/"
            "(?P<namespace>([^/]*/)*[^/]*)$"
        ),
        re.IGNORECASE,
    )
    captured = re.match(regex, url).groupdict()
    base, namespace = [captured[group] for group in ["base", "namespace"]]
    return base, namespace


def get_project_id(project_url: str, header: Dict[str, str]) -> int:
    """Given a project's URL, return it's gitlab ID"""
    base, namespace, repo = parse_repo_url(project_url)
    project = []
    page = 1
    # Gitlab only provides max 100 results/page, we need to scan all pages
    # in case the project is not on the first one
    while not len(project):
        resp = requests.get(
            f"{base}/api/v4/projects?search={repo}&per_page=100&page={page}",
            headers=header,
        )
        if resp.ok:
            resp = resp.json()
        else:
            resp.raise_for_status()
        page += 1
        project = [p for p in resp if p["path_with_namespace"] == f"{namespace}/{repo}"]

    if len(list(project)) > 1:
        raise ValueError("More than one project matched input url")
    return project[0]["id"]


def get_group_id(group_url: str, header: Dict[str, str]) -> int:
    """Given a group's URL, return it's gitlab ID"""
    base, namespace = parse_group_url(group_url)
    if namespace.find("/") > 0:
        parent_name = namespace.split("/")[0]
        group_name = namespace.split("/")[-1]
        query = f"{base}/api/v4/groups/{parent_name}/subgroups?search={group_name}"
    else:
        query = f"{base}/api/v4/groups/{namespace}"

    response = requests.get(query, headers=header)
    if response.ok:
        content = response.json()
        # Subgroup response comes as a list and must be unwrapped
        if isinstance(content, list):
            content = content[0]
        return content["id"]
    else:
        response.raise_for_status()
