# Given an input csv file of course participants created from moodle,
# create corresponding groups in moodle.

import json
import datetime
from typing import Dict
import requests
import pandas as pd
import click
from teach_auto.common_requests import get_group_id, parse_group_url
from teach_auto.utils import ingest_token


def create_student_group(
    group_name: str, parent_url: str, header: Dict[str, str]
) -> requests.Response:
    """Create a private group for student"""
    data = {
        "path": group_name,
        "name": group_name,
        "parent_id": get_group_id(parent_url),
        "visibility": "private",
    }
    base, _ = parse_group_url(parent_url)
    response = requests.post(
        f"{base}/groups",
        data=json.dumps(data),
        headers={**header, "Content-Type": "application/json"},
    )
    return response


def invite_student_email(
    student_email: str, base_url: str, group_id: int, header: Dict[str, str]
) -> None:
    """Invite student to group by email"""
    response = requests.post(
        f"{base_url}/groups/{group_id}/invitations",
        data={"email": student_email, "access_level": 50},
        headers=header,
    )
    print(response.json())


def generate_student_code(first_name: str, last_name: str) -> str:
    """Generate a uniform student identifier from their name"""
    initials = first_name[0] + last_name[0]
    year = datetime.datetime.now().year
    return f"ST_{initials}_{year}".upper()


@click.command()
@click.argument("student_table")
@click.argument("parent_url")
@click.option(
    "--token-path",
    type=click.Path(exists=True),
    help="Path to a file containing the Gitlab API token. If not provided, you will be prompted for the token.",
)
def create_groups(student_table, parent_url, token_path):
    """Given a parent-group URL and a CSV file of students exported from Moodle,
    create one private subgroup for each student in the parent group and invite students
    in their personal group. The student table must contain columns: "Email address", "First name"
    and "Surname"
    """
    token = ingest_token(token_path)
    header = {"PRIVATE-TOKEN": token}

    students = pd.read_csv(student_table)
    base_url, _ = parse_group_url(parent_url)

    for _, st in students.iterrows():
        # Generate a student code, e.g.: Ben Smith -> ST_BS_2022
        student_code = generate_student_code(st["First name"], st["Surname"])

        # Create student group "parent-group/ST_BS_2022"
        resp = create_student_group(student_code, parent_url, header)
        # Skip if already created
        if not resp.ok:
            print(f"Could not create {student_code}: {resp.json()['message']}")

        group_id = get_group_id(f"{parent_url}/{student_code}", header)
        # Invite student to their group by email
        invite_student_email(st["Email address"], base_url, group_id, header)


if __name__ == "__main__":
    create_groups()
